class_name RingBuffer extends RefCounted

enum Direction {
    Left = -1,
    Right = 1
}

signal updated(new_range: IntRange, previous_range: IntRange)

var _source_size: int
var source_size: int:
    get: return _source_size
    set(value): set_source_size(value)

func set_source_size(value: int):
    assert(value >= 0, "source size must be a non-negative integer")
    _source_size = value

var _capacity: int
var capacity: int:
    get: return _capacity
    set(value): set_capacity(value)

func set_capacity(value: int):
    _capacity = value

var _index: int

var selected_index: int:
    get: return _index
    set(value): set_selected(value)

func set_selected(value: int):
    _index = self.wrap(value)

var start_index: int:
    get: return _index

var end_index: int:
    get: return self.wrap(_index + _capacity - 1)

@warning_ignore("shadowed_variable")
func _init(source_size: int, capacity: int, start_index: int = 0) -> void:
    self.source_size = source_size
    self.capacity = capacity
    selected_index = start_index

func wrap(value: int) -> int:
    if _source_size == 0:
        return 0
    else:
        return value % _source_size

func range() -> RangeIterator:
    return RangeIterator.new(IntRange.new(start_index, end_index))

func move(direction: Direction) -> Result:
    if _source_size <= _capacity:
        return Result.err("source size is smaller than buffer capacity")
    var previous = range()
    _index = self.wrap(_index + direction)
    updated.emit(range(), previous)
    return Result.Unit

func move_right() -> Result:
    return move(Direction.Right)

func move_left() -> Result:
    return move(Direction.Left)
